@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2009-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -20,11 +20,12 @@
 import uuid
 import logging
 
-from datetime import timedelta
 from email.utils import formataddr
 from enum import Enum
+from lazr.config import as_boolean
 from mailman.app.membership import delete_member
 from mailman.app.workflow import Workflow
+from mailman.config import config
 from mailman.core.i18n import _
 from mailman.database.transaction import flush
 from mailman.email.message import UserNotification
@@ -33,14 +34,23 @@ from mailman.interfaces.bans import IBanManager
 from mailman.interfaces.listmanager import ListDeletingEvent
 from mailman.interfaces.mailinglist import SubscriptionPolicy
 from mailman.interfaces.member import (
-    AlreadySubscribedError, MemberRole, MembershipIsBannedError,
-    NotAMemberError)
+    AlreadySubscribedError,
+    DeliveryMode,
+    DeliveryStatus,
+    MemberRole,
+    MembershipIsBannedError,
+    NotAMemberError,
+)
 from mailman.interfaces.pending import IPendable, IPendings
 from mailman.interfaces.subscriptions import (
-    ISubscriptionManager, ISubscriptionService,
-    SubscriptionConfirmationNeededEvent, SubscriptionInvitationNeededEvent,
-    SubscriptionPendingError, TokenOwner,
-    UnsubscriptionConfirmationNeededEvent)
+    ISubscriptionManager,
+    ISubscriptionService,
+    SubscriptionConfirmationNeededEvent,
+    SubscriptionInvitationNeededEvent,
+    SubscriptionPendingError,
+    TokenOwner,
+    UnsubscriptionConfirmationNeededEvent,
+)
 from mailman.interfaces.template import ITemplateLoader
 from mailman.interfaces.user import IUser
 from mailman.interfaces.usermanager import IUserManager
@@ -155,7 +165,8 @@ class _SubscriptionWorkflowCommon(Workflow):
             when=now().replace(microsecond=0).isoformat(),
             token_owner=token_owner.name,
             )
-        self.token = pendings.add(pendable, timedelta(days=3650))
+        # MAS pendings.add will set lifetime based on token_owner.
+        self.token = pendings.add(pendable)
 
 
 @public
@@ -174,11 +185,14 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         'user_key',
         'token_owner_key',
         'send_welcome_message',
+        'delivery_mode',
+        'delivery_status',
         )
 
     def __init__(self, mlist, subscriber=None, *,
                  pre_verified=False, pre_confirmed=False, pre_approved=False,
-                 invitation=False, send_welcome_message=None):
+                 invitation=False, send_welcome_message=None,
+                 delivery_mode=None, delivery_status=None):
         super().__init__(mlist, subscriber)
         # An invitation is already supposed to be "pre_approved" and by
         # confirming the invite a user will "confirm" and "verify" their
@@ -189,6 +203,18 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         self.pre_approved = invitation or pre_approved
         self.invitation = invitation
         self.send_welcome_message = send_welcome_message
+        # For enum types, we use the string values here instead of the enum
+        # objects because they are easier to serialize when the workflow is
+        # saved in the database. When setting the values, we restore back to
+        # Enum types.
+        if delivery_status is not None:
+            self.delivery_status = delivery_status.name
+        else:
+            self.delivery_status = None
+        if delivery_mode is not None:
+            self.delivery_mode = delivery_mode.name
+        else:
+            self.delivery_mode = None
 
     def _step_sanity_checks(self):
         # Ensure that we have both an address and a user, even if the address
@@ -211,17 +237,10 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         assert self.user is not None and self.address is not None, (
             'Insane sanity check results')
         # Is this subscriber already a member?
-        if (self.which is WhichSubscriber.user and
-                self.user.preferred_address is not None):
-            subscriber = self.user
-        else:
-            subscriber = self.address
-        if self.mlist.is_subscribed(subscriber):
-            # 2017-04-22 BAW: This branch actually *does* get covered, as I've
-            # verified by a full coverage run, but diffcov for some reason
-            # claims that the test added in the branch that added this code
-            # does not cover the change.  That seems like a bug in diffcov.
-            raise AlreadySubscribedError(           # pragma: nocover
+        if ((self.user.preferred_address is not None and
+                self.mlist.is_subscribed(self.user)) or
+                self.mlist.is_subscribed(self.address)):
+            raise AlreadySubscribedError(
                 self.mlist.fqdn_listname,
                 self.address.email,
                 MemberRole.member)
@@ -307,8 +326,8 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         # Possibly send a notification to the list moderators.
         if self.mlist.admin_immed_notify:
             subject = _(
-                'New subscription request to $self.mlist.display_name '
-                'from $self.address.email')
+                'New subscription request to ${self.mlist.display_name} '
+                'from ${self.address.email}')
             username = formataddr(
                 (self.subscriber.display_name, self.address.email))
             template = getUtility(ITemplateLoader).get(
@@ -342,6 +361,13 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         # We can immediately subscribe the user to the mailing list.
         self.member = self.mlist.subscribe(
             self.subscriber, send_welcome_message=self.send_welcome_message)
+        # Set member attributes.
+        if self.delivery_mode:
+            self.member.preferences.delivery_mode = DeliveryMode[
+                self.delivery_mode]
+        if self.delivery_status:
+            self.member.preferences.delivery_status = DeliveryStatus[
+                self.delivery_status]
         assert self.token is None and self.token_owner is TokenOwner.no_one, (
             'Unexpected active token at end of subscription workflow')
 
@@ -468,8 +494,8 @@ class UnSubscriptionWorkflow(_SubscriptionWorkflowCommon):
             self.mlist.fqdn_listname, self.address.email))
         if self.mlist.admin_immed_notify:
             subject = _(
-                'New unsubscription request to $self.mlist.display_name '
-                'from $self.address.email')
+                'New unsubscription request to ${self.mlist.display_name} '
+                'from ${self.address.email}')
             username = formataddr(
                 (self.subscriber.display_name, self.address.email))
             template = getUtility(ITemplateLoader).get(
@@ -541,7 +567,8 @@ class SubscriptionManager:
 
     def register(self, subscriber=None, *,
                  pre_verified=False, pre_confirmed=False, pre_approved=False,
-                 invitation=False, send_welcome_message=None):
+                 invitation=False, send_welcome_message=None,
+                 delivery_mode=None, delivery_status=None):
         """See `ISubscriptionManager`."""
         workflow = SubscriptionWorkflow(
             self._mlist, subscriber,
@@ -549,7 +576,10 @@ class SubscriptionManager:
             pre_confirmed=pre_confirmed,
             pre_approved=pre_approved,
             invitation=invitation,
-            send_welcome_message=send_welcome_message)
+            send_welcome_message=send_welcome_message,
+            delivery_mode=delivery_mode,
+            delivery_status=delivery_status,
+            )
         list(workflow)
         return workflow.token, workflow.token_owner, workflow.member
 
@@ -593,16 +623,17 @@ def _handle_confirmation_needed_events(event, template_name):
     # This function handles sending the confirmation email to the user
     # for both subscriptions requiring confirmation and invitations
     # requiring acceptance.
-    if template_name.endswith(':invite'):
-        subject = _('You have been invited to join the '
-                    '$event.mlist.fqdn_listname mailing list.')
-    elif template_name.endswith(':unsubscribe'):
-        subject = _('Your confirmation is needed to leave the '
-                    '$event.mlist.fqdn_listname mailing list.')
-    else:
-        assert(template_name.endswith(':subscribe'))
-        subject = _('Your confirmation is needed to join the '
-                    '$event.mlist.fqdn_listname mailing list.')
+    with _.using(event.mlist.preferred_language.code):
+        if template_name.endswith(':invite'):
+            subject = _('You have been invited to join the '
+                        '${event.mlist.fqdn_listname} mailing list.')
+        elif template_name.endswith(':unsubscribe'):
+            subject = _('Your confirmation is needed to leave the '
+                        '${event.mlist.fqdn_listname} mailing list.')
+        else:
+            assert template_name.endswith(':subscribe')
+            subject = _('Your confirmation is needed to join the '
+                        '${event.mlist.fqdn_listname} mailing list.')
     confirm_address = event.mlist.confirm_address(event.token)
     email_address = event.email
     # Send a verification email to the address.
@@ -618,6 +649,10 @@ def _handle_confirmation_needed_events(event, template_name):
         domain_name=event.mlist.domain.mail_host,
         contact_address=event.mlist.owner_address,
         ))
+    if ('verp_confirmations' in config.mta and not
+            as_boolean(config.mta.verp_confirmations)):
+        subject = 'confirm {}'.format(event.token)
+        confirm_address = event.mlist.request_address
     msg = UserNotification(
         email_address, confirm_address, subject, text,
         event.mlist.preferred_language)
